#define _LARGEFILE64_SOURCE
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <signal.h>

#include <stdio.h>
#include <time.h>

#include "error.h"

void sigh(int sig)
{
	FILE *fh = fopen("/proc/sys/vm/drop_caches", "w");
	if (!fh)
		error_exit("Failed to open /proc/sys/vm/drop_caches");
	fprintf(fh, "0\n");
	fclose(fh);

	exit(1);
}

void flush_cache()
{
	FILE *fh = fopen("/proc/sys/vm/drop_caches", "w");
	if (!fh)
		error_exit("Failed to open /proc/sys/vm/drop_caches");
	fprintf(fh, "3\n");
	fclose(fh);
}

int main(int argc, char *argv[])
{
	time_t start = time(NULL), start2 = start;
	long long int cnt = 0, size;
	int fd;
	fprintf(stderr, "iops v" VERSION ", (C) 2009 by folkert@vanheusden.com\n\n");

	signal(SIGINT, sigh);
	signal(SIGTERM, sigh);

	if (argc != 3)
		error_exit("usage: %s device sizeinbytes\n", argv[0]);

	fd = open(argv[1], O_RDONLY);
	if (fd == -1)
		error_exit("cannot open %s", argv[1]);

	size = atoll(argv[2]);

	flush_cache();

	for(;;)
	{
		time_t now = time(NULL);
		char buffer[4096];
		off64_t off;
		int diff;

		do
		{
			off = ((off64_t)lrand48()) << (off64_t)30 + (off64_t)lrand48();
			off %= (size - (off64_t)4096);
		}
		while(off < 0 || off >= size);

		off &= (off64_t)~((off64_t)4095);

		if (lseek64(fd, off, SEEK_SET) == (off64_t)-1)
			error_exit("seek_set failed");

		if (read(fd, buffer, 4096) != 4096)
			error_exit("error reading from disk");

		cnt++;

		diff = now - start2;
		if (diff % 5 == 0 && diff > 0)
		{
			flush_cache();

			printf("%f %lld\n", (double)cnt / (double)(now - start), cnt);

			start2 = now;
		}
	}

	close(fd);

	return 0;
}
